% Fatigue calculations from FEDEM input file
% ======================================================
clc; clear all; close all
% Fatigue calculations for a monopil, only considering the loads below
% mudline.
% s_ind     -   time step to start rainflow counting - manual input
% t_ind     -   time step to end rainflow counting
% noLC      -   number of loads case run in FEDEM - manual input
% triad     -   triad number in FEDEM - manual input
% z_triad   -   z-position 
% d_o       -   outer diameter of pile
% d_i       -   inner diameter of pile
% m         -   constant used for the S-N curve in DNV RP C203
% loga      -   constant used for the S-N curve in DNV RP C203
% stresslim -   split of the 2 comp SN curve in DNV RP C203
% tref      -   reference thickness from DNV RP C203
% k         -   thickness exponent
% LPorder   -   Butterworth order in filter
% LPcut     -   Cut off frequency used by filter  [Hz]
% I_pile    -   second moment of inertia of the pile
% thk       -   thickness of pile wall
% noStress  -   number of measured stress points
% DRFC      -   matrix contatining damage 
% filename  -   filename for the load case
% fid       -   file identifier
% a         -   matrix containing the raw data from file 'filename'
% time      -   time steps
% rot_speed -   rotational speed
% moment_y  -   moment about the y axis (fore-aft bending)
% stress_raw-   raw data of axial stress
% stress    -   axial stress after filtration
% x         -   matrix with time in 1st column and stress in remaining
% tp        -   turning points of the stress series
% def       -   definitions used in the rainfolw counting
% RFC       -   rainflow counting matrix
% 
% -----------------------------------------------------------
% INPUT 
% ---------------------------------------------------------

noLC=13;     % number of load cases
triads=[9];   % triad number in FEDEM model
z_triads=[-45.0];   % location relative to LAT. seabed at -28.4      
% cross-section pile
d_o=8;
d_i=7.879;
% S-N curve, D tab 2-2 DNV-RP-C203
m=[3 5];
loga=[11.764 15.606];
stresslim=52.63;
tref=0.025;
k=0.2;          % thickness exponent
%% filter
LPorder=8;
LPcut=0.6;
%% ------------------------------------------------------------
% geometric sizes
I_pile=pi*((d_o/2)^4-(d_i/2)^4)/4 % second moment of inertia pile
thk=d_o/2-d_i/2            % thickness
noStress=length(triads);    % number of measured stress points
% ------------------------------------------------------------
% empty matrices
DRFC=zeros(noStress,noLC);
%%

for lc=1:noLC
    % ==========================================================
    % Read in file
    % ==========================================================
    % the first 8 rows are headerlines
    % the file should have the following columns
    % time [s]          - c1
    % rot speed [rad/s] - c2
    % pile M_y [Nm]     - c3 -> c15
    % wave elevation[m] - c16
    % wind speed [m/s]  - c17
    % tower, M_y [Nm]   - c18 -> c28
    filename=['/Users/TineLouiseTrooen/Dropbox/Tine (4)/Tine_M_springing_event_' num2str(lc) '.asc'];
    fid=fopen(filename);
    a=textscan(fid,'%f %f %f %f %f','headerLines',7);
    fclose(fid);
    time=a{1};
%     rot_speed=a{4};
    moment_y=[a{2}];
    s_ind=1/(time(2)-time(1));
    e_ind=length(moment_y);      
    stress_raw=moment_y.*d_o/2/I_pile/1e6;  % [MPa]
%     wind=a{17};
%     wave=a{16};
    %% filter the stress
    for ik=1:noStress
        %%
        [zb,pb]=butter(LPorder,LPcut,'low');
        [stress(:,ik)]=filter(zb,pb,stress_raw(:,ik)');
        figure()
        plot(time(8000:296000),stress(8000:296000,ik),time(8000:296000),stress_raw(8000:296000,ik))
        legend('filtered','raw')
        %%
    end
    time=time(s_ind:e_ind);    
%     rot_speed=rot_speed(s_ind:e_ind);
    stress=stress(s_ind:e_ind,1:noStress); 
    %% ==========================================================
    % Rainflow counting
    % ==========================================================
    for ip=1:noStress      % no of points where stresses are measured
        x=[time(7922:295922) stress(7922:295922,ip)]; %Stress starts indicing at t=1sec, meaning at time step number 80 
        [tp]=dat2tp(x);     %turning points
        def.res='CS';
        def.asymmetric=0;
        def.time=0;
        [RFC]=tp2rfc(tp,def);      % rainflow cycles
%         figure()
%         ccplot(RFC)
%         title(['Rainflow plot, triade ' num2str(triads(ip)) ' at ' num2str(z_triads(ip)) ' m rel to LAT'])
        % output bins
        % needs input
        % damage script
        DRFC2(ip,lc)=cc2dam_lene(RFC,m,loga,stresslim,tref,thk,k);    %damage calculation
        clear x 
        
    end
    clear a filename stress stress_raw moment_y
end
%% Plotting fatigue damage
figure()
bar(DRFC, 'b')
hold on 
bar(DRFC2, 'y')
%legend('Hs: 2.50 m, Tp: 6.08')
title('Fatigue damage')
legend('Fatigue damage second order FNV','Fatigue damage Morison' )
xlabel('Load Case number')
ylabel('Damage for 1 hour simulation')

DRFC
