	function [Sf,ff]=SPEGEN_T(xt,NB,dt,IOPT)
% function [Sf,ff]=spegen_t(xt,NB,dt,IOPT)
% Routine for generating power spectrum from the time history xt.
%  NB is the number of blocks the time history is splitted into.
% The sampling interval is dt.
% Finn Gunnar Nielsen , 6 Dec. 1995
% last revision  15 April 2011: IOPT for plotting / printing included
%
% Input values
% Time history                    : xt (e.g. m)
% Number of blocks in analysis    : NB
% Sampling interval               : dt (sec)
% IOPT                            : 0: No print of plotting, =1: Printing
% only, = 2: Printing and plotting
%
% Output values			
% Power pectral values            : S(f) (e.g. m^2*sec)
% Frequncies where S(f) are given : ff (1/sec)
%
% S(f) is the one-sided power spectrum as function of frequency.
% It is estimated by splitting the time history up in NB blocks 
% of length TB. 
% An FFT is applied on each block and the results are averaged. 
% Each block is smoothed by a cosine function at each
% end to obtain smooth estimates. 
%
% Statistical estimates from each block are computed. Average values
% for the complete time history as well as standard deviation of the 
% quantities are established.
%
% Default values
% Length of smoothing at each end of the blocks, sl=bl/10. 
% Correction factor on power spectrum due to cosine smooting: 
% 1/0.875 ( = 1/[0.8 + 0.2*int((0.5*(1+cos(pi*x/L)))^2 dx].

% Length of each block
% Make input a column vector
[m0,n0] = size(xt);
if m0==1,    
    xt = xt';                              % If a row vector, make x a column vector
else                          
end
NT = size(xt,1);
bl = 2*floor(0.5*NT/NB);

sll = 10;                   % Fraction of each block that is involved in smoothing
corr = 1/0.875;             % Correction factor to compensate the effect of smoothing

% Main parameters
tmax =  (bl*NB)*dt;
df = NB/tmax;
ff = (df:df:(bl/2)*df)';

% Zeroing matrices
Y = zeros(bl,NB);
Pyy = zeros(bl/2,NB);
xb = zeros(bl,1)';

% clearing variables
clear xmean area area02 Hsig t02 xmtt sigt;

%Smooting function;
sm = ones(bl,1);
n1 = floor(bl/sll);
for k = 1:n1
	sfac = 0.5*(1 - cos(pi*k/n1));
	sm(k,1) = sfac;
	sm(bl-k+1,1) = sfac;
end

% Loop over the blocks
for k=1:NB
	kmin = (k-1)*bl +1;
	kmax = kmin + bl-1;
	xmt(k) = mean(xt(kmin:kmax,1));
	xb = (xt(kmin:kmax,1)-xmt(k)).*sm;

% FFT and power spectrum
	Y(:,k) = fft(xb,bl);
	cor1 = corr*dt/bl;
	Pyy(:,k) = 2.*Y(1:bl/2,k).*conj(Y(1:bl/2,k)).*cor1;

% Area and moments
% Y(1,k) is the mean value of the time series.

	xmean(k) = Y(1,k)'/bl+xmt(k);
	area(k) = df*trapz(Pyy(1:bl/2,k))';
	area02(k) = df*trapz(Pyy(:,k).*ff.^2)';
	Hsig(k) = 4.*sqrt(area(k))';
   t02(k) = 1./(sqrt(area02(k)/area(k)))';
   peak(k) = max(Pyy(2:bl/2,k));
   ind_peak = find(Pyy(2:bl/2,k)>(peak(k)-peak(k)/10^8));
   tpeak(k) = 1./ff(ind_peak);

end

% total estimates
Pyy = Pyy';
if NB>1
	Sf = (sum(Pyy)/NB)';
	else
	Sf=Pyy;
	end
xtmean = sum(xmean)/NB;
sigxm = std(xmean);
m0 = sum(area)/NB;
sigm0 = std(area);
meanHs = sum(Hsig)/NB;
T02 = sum(t02)/NB;
sigt02 = std(t02);
tpeakmean=sum(tpeak)/NB;
sigtpeak = std(tpeak);

% Estimates from time history
xmtt = mean(xt(1:NT));
sigt = std(xt(1:NT));
maxxt = max(xt(1:NT));
minxt = min(xt(1:NT));

if IOPT > 0
s = ['Length of record,           Tmax = ',num2str((NT)*dt)];
disp(s);
s = ['Length of rec. used in FFT, T_used = ',num2str(tmax)];
disp(s);
s = ['Timestep,                     dt = ',num2str(dt)];
disp(s);
s = ['Number of blocks in record,   NB = ',int2str(NB)];
disp(s);
s=[' '];
disp(s);
s=['Values obtained directly from time history'];
disp(s);
s = ['Mean value of record,      xmean  = ',num2str(xmtt)];
disp(s);
s = ['Standard deviation,      sig(xt)  = ',num2str(sigt)];
disp(s);
s = ['Maximum value in record, Max(xt)  = ',num2str(maxxt)];
disp(s);
s = ['Minimum value in record, Min(xt)  = ',num2str(minxt)];
disp(s);
s=[' '];
disp(s);
s=['Values obtained from spectrum'];
disp(s);
s = ['Mean value of record,       xmean = ',num2str(xtmean)];
disp (s);
s = ['Frequency increment,           df = ',num2str(df)];
disp (s);
s = ['Stand. dev. of mean,    sig(mean) = ',num2str(sigxm)];
disp(s); 
s = ['Area of spectrum,              m0 = ',num2str(m0)];
disp (s);
s = ['Standard deviation,       sig(xt) = ',num2str(m0^(1/2))];
disp(s);
s = ['Stand. dev. of area,      sig(m0) = ',num2str(sigm0)];
disp(s); 
s = ['Significant value,      4*sig(xt) = ',num2str(4.*sqrt(m0))];
disp(s);
s = ['Aver, zero upcrossing per,    T02 = ',num2str(T02),' (sec)'];
disp(s);
s = ['Standard dev. of T02,    sig(T02) = ',num2str(sigt02),' (sec)'];
disp (s);
s = ['Aver, peak period,             Tp = ',num2str(tpeakmean),' (sec)'];
disp(s);
s = ['Standard dev. of Tp,      sig(Tp) = ',num2str(sigtpeak),' (sec)'];
disp (s);
s = ['Mean peak value         , S(1/Tp) = ',num2str(max(Sf))];
disp (s);
end
if IOPT > 1
figure
plot(ff(2:bl/2),Sf(2:bl/2),'k-');
grid;
s = ['Spectrum, m0= ',num2str(m0),' '];
title(s);
xlabel(' Frequency (1/sec)');
ylabel(' S(f) ');
end




